package edu.gatech.cercs.soa.gds.web;


import java.rmi.RemoteException;

import javax.jms.*;
import javax.naming.InitialContext;

import edu.gatech.cercs.soa.core.*;
import edu.gatech.cercs.soa.core.fpq.IHeader;
import edu.gatech.cercs.soa.core.fpr.IFlightPricingResult;
import edu.gatech.cercs.soa.gds.FlightPricingState;

/**
 * Service endpoint implementation for FlightPricingService.
 * 
 * @author <a href="mailto:srihari@cc.gatech.edu">Hari Govindharaj</a>
 */
public class FlightPricingServiceServlet implements FlightPricingService {

    private static final String PROPERTIES_FILE = "gds.properties";    

    private static final String CONNECTION_FACTORY_NAME = "jms.connection";
    private static final String QUEUE_NAME = "jms.queue";
    
    /**
     * The searchPrices call submits the query string in XML format for GDS
     * processing.
     * @param query - non-null string in XML format
     * @return String - contains status info in XML format
     * @throws RemoteException
     */
    public String searchPrices(String query) throws RemoteException {
        IFlightPricingResult fpr = null;
        
        //Parse the header part of the query
        IHeader hdr = FlightPricingUtils.parseQueryHeader(query);
        if (hdr == null) {
            //Header part of query missing? Return error message.
            fpr = FlightPricingFactory.createErrorResult(
                    "Error parsing query header.");
        }
        else {
            try {            
                //Post the query to a configured queue.
                sendToQueue(query);
                fpr = FlightPricingFactory.createQueueResult("Queued for QPS.");
            }
            catch (Exception e) {
                e.printStackTrace(); 
                fpr = FlightPricingFactory.createErrorResult(e.getMessage());
            }
        }
        
        FlightPricingUtils.updateResultHeader(hdr, fpr);
        FlightPricingState.getInstance().updateResult(fpr);
        return FlightPricingUtils.dumpResult(fpr);
    }

    private void sendToQueue(String query) throws Exception {
        InitialContext context = null;
        Connection connection = null;
        MessageProducer producer = null;
        Session session = null;

        try {
            PropertyLoader propLoader = 
                PropertyLoader.getInstance(PROPERTIES_FILE);
            context = new InitialContext();
    
            ConnectionFactory factory = (ConnectionFactory) context
                        .lookup(propLoader.getValue(CONNECTION_FACTORY_NAME));
            Queue queryQueue = (Queue)context.lookup(propLoader
                        .getValue(QUEUE_NAME));
            connection = factory.createConnection();
            session = connection.createSession(false,
                    Session.AUTO_ACKNOWLEDGE);
            connection.start();
    
            producer = session.createProducer(queryQueue);
            TextMessage textMessage = session.createTextMessage();
            textMessage.setText(query);
            producer.send(textMessage);
        }
        finally {
            try {
                if (producer != null)
                    producer.close();
                if (session != null)
                    session.close();
                if (connection != null)
                    connection.close();
            }
            catch (JMSException e) {
                e.printStackTrace(); //TODO - log and return an error string
            }
        }
    }
    
    /**
     * The retrieveResult call searches the flight pricing state table for the
     * query result.
     * @param queryId - non-null string
     * @return String - contains status info or query result in XML format
     * @throws RemoteException
     */
    public String retrieveResult(String customerId, String queryId) 
            throws RemoteException {
        if (FlightPricingState.getInstance().hasResult(customerId, queryId)) {
            return FlightPricingState.getInstance().getResultXml(customerId, queryId);
        }
        else {
            throw new RemoteException("Unrecognized customer and query Identifiers.");
        }
    }
}
