package edu.gatech.cercs.soa.tws.web;


import java.io.*;
import java.net.*;
import java.util.Date;

import javax.servlet.ServletException;
import javax.servlet.http.*;
import javax.xml.namespace.QName;
import javax.xml.rpc.Service;
import javax.xml.rpc.ServiceFactory;

import edu.gatech.cercs.soa.core.*;
import edu.gatech.cercs.soa.core.fpq.*;
import edu.gatech.cercs.soa.core.fpr.IFlightPricingResult;
import edu.gatech.cercs.soa.core.fpr.IHeaderResult;
import edu.gatech.cercs.soa.gds.web.FlightPricingService;
import edu.gatech.cercs.soa.tws.FlightPricingState;


/**
 * Describe <code>FlightPricingSearchServlet</code> here.
 *
 * @author <a href="mailto:srihari@cc.gatech.edu">Hari Govindharaj</a>
 */
public class FlightPricingSearchServlet extends HttpServlet {
   
   private static final long serialVersionUID = 1L;

   private static final String SERVICE_TWS_URL = "service.tws.URL";
   private static final String SERVICE_TWS_NAMESPACE_URI = "service.tws.NamespaceURI";
   private static final String SERVICE_TWS_NAME = "service.tws.Name";
   private static final String SERVICE_TWS_PORT = "service.tws.Port";

   private static final String SERVICE_GDS_URL = "service.gds.URL";
   private static final String SERVICE_GDS_NAMESPACE_URI = "service.gds.NamespaceURI";
   private static final String SERVICE_GDS_NAME = "service.gds.Name";
   
   private static final String PROPERTIES_FILE = "tws.properties";

   private static PropertyLoader propLoader;
   private static String customerId;

   static {
      propLoader = PropertyLoader.getInstance(PROPERTIES_FILE);
      customerId = propLoader.getValue(SERVICE_TWS_URL) + "::" +
         propLoader.getValue(SERVICE_TWS_NAMESPACE_URI) + "::" +
         propLoader.getValue(SERVICE_TWS_NAME) + "::" +
         propLoader.getValue(SERVICE_TWS_PORT);
   }
   
   public void doGet(HttpServletRequest request, HttpServletResponse response)
         throws ServletException, IOException {
      String queryId = request.getParameter("queryId");
      if (queryId != null && queryId.length() > 0) {
         queryId = URLDecoder.decode(queryId, "UTF-8");
         if (FlightPricingState.getInstance().hasResult(customerId, queryId)) {

            IFlightPricingResult fpr = FlightPricingState.getInstance().
               getResult(customerId, queryId);
            
            //Check if we have got the result back from GDS
            if (fpr.getHeader().getStatus().equalsIgnoreCase(IHeaderResult.STATUS_COMPLETE)) {
               request.setAttribute("fpr", fpr);
               getServletContext().getRequestDispatcher("/jsp/results.jsp").
                  forward(request, response);
               return;
            }
            else if (fpr.getHeader().getStatus().equalsIgnoreCase(IHeaderResult.STATUS_QUEUE)) {
               request.setAttribute("queryId", queryId);
               getServletContext().getRequestDispatcher("/jsp/status.jsp").
                  forward(request, response);
               return;
            }
         }
      }
      getServletContext().getRequestDispatcher("/jsp/index.jsp").
         forward(request, response);
   }
   
   public void doPost(HttpServletRequest request, HttpServletResponse response)
         throws ServletException, IOException {
      
      String from = request.getParameter("from");
      String to = request.getParameter("to");
      String dep = request.getParameter("depart");
      String ret = request.getParameter("return");
      
      Date depDate = DateUtils.parse(dep, DateUtils.standardSDF); // mm/dd/yyyy
      Date retDate = DateUtils.parse(ret, DateUtils.standardSDF);
      
      IFlightPricingQuery fpQuery = FlightPricingFactory.createQuery();
      
      IHeader hdr = fpQuery.getHeader();
      hdr.setCustomerId(customerId);
      hdr.setQueryMode(IHeader.QUERYMODE_ASYNC);
      hdr.setExpiration(15000);
      hdr.setTimestamp(System.currentTimeMillis());
      
      //Onward journey
      ITrip onwardTrip = fpQuery.getItinerary().createTrip();
      onwardTrip.setOrigin(from);
      onwardTrip.setDestination(to);
      onwardTrip.setDate(depDate);
      onwardTrip.setNonStop(true);
      fpQuery.getItinerary().addTrip(onwardTrip);
      
      //Backward journey
      ITrip backwardTrip = fpQuery.getItinerary().createTrip();
      backwardTrip.setOrigin(to);
      backwardTrip.setDestination(from);
      backwardTrip.setDate(retDate);
      backwardTrip.setNonStop(true);
      fpQuery.getItinerary().addTrip(backwardTrip);      

      //TODO - add Fareclass, Passengers and Airline params to the query.

      try {
         send(fpQuery);
         request.setAttribute("queryId", 
                  URLEncoder.encode(fpQuery.getHeader().getQueryId(), "UTF-8"));
         getServletContext().getRequestDispatcher("/jsp/status.jsp").
            forward(request, response);
      }
      catch (Exception ex) {
         //TODO - redirect to an error page
         ex.printStackTrace();
         getServletContext().getRequestDispatcher("/jsp/index.jsp").
            forward(request, response);
      }
   
   }
   
   private void send(IFlightPricingQuery fpQuery) throws Exception {
      //Set the status of the query in the FlightPricingState table.
      IFlightPricingResult fpResult = 
         FlightPricingFactory.createQueueResult("Query Sent to GDS.");
      FlightPricingUtils.updateResultHeader(fpQuery, fpResult);
      FlightPricingState.getInstance().updateResult(fpResult);
      
      //Send the query to the GDS Service
      URL url = new URL(propLoader.getValue(SERVICE_GDS_URL));
      QName qname = new QName(propLoader.getValue(SERVICE_GDS_NAMESPACE_URI), 
               propLoader.getValue(SERVICE_GDS_NAME));
      ServiceFactory factory = ServiceFactory.newInstance();
      Service service = factory.createService(url, qname);
      FlightPricingService fpService = (FlightPricingService) service
               .getPort(FlightPricingService.class);
      
      StringWriter sw = new StringWriter();
      fpQuery.saveAsXML(new PrintWriter(sw));
      String status = fpService.searchPrices(sw.toString());

      //Update the query status in the FlightPricingState table.
      FlightPricingState.getInstance().updateResult(status);      
   }
     
}